# Generated by Django 3.0.8 on 2020-07-12 20:48

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="LeadSource",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "medium",
                    models.CharField(
                        help_text="utm_medium: Identifies what type of link was used, such as cost per click or email.",
                        max_length=10,
                    ),
                ),
                (
                    "source",
                    models.CharField(
                        help_text="utm_source: Identifies which site sent the traffic, and is a required parameter.",
                        max_length=30,
                    ),
                ),
                (
                    "campaign",
                    models.CharField(
                        blank=True,
                        help_text="utm_campaign: Identifies a specific product promotion or strategic campaign.",
                        max_length=100,
                    ),
                ),
                (
                    "term",
                    models.CharField(
                        blank=True,
                        help_text="utm_term: Identifies search terms.",
                        max_length=50,
                    ),
                ),
                (
                    "content",
                    models.CharField(
                        blank=True,
                        help_text="utm_content: Identifies what specifically was clicked to bring the user to the site, such as a banner ad or a text link.",
                        max_length=50,
                    ),
                ),
                (
                    "timestamp",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="When the event occurred.",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="lead_sources",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={"get_latest_by": ("timestamp",),},
        ),
    ]



# Generated by Django 3.0.8 on 2020-07-13 09:48

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("utm_tracker", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="leadsource",
            name="created_at",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                help_text="When the event was recorded.",
            ),
        ),
    ]



import logging
from typing import Callable, Dict

from django.http import HttpRequest, HttpResponse

from .models import LeadSource

logger = logging.getLogger(__name__)


class UtmSessionMiddleware:
    """Extract utm values from querystring and store in session."""

    def __init__(self, get_response: Callable[[HttpRequest], HttpResponse]):
        self.get_response = get_response

    def __call__(self, request: HttpRequest) -> HttpResponse:
        request.session.update(self._utm(request))
        return self.get_response(request)

    def _utm(self, request: HttpRequest) -> Dict[str, str]:
        """
        Extract 'utm_*' values from request querystring.

        NB in the case where there are multiple values for the same key,
        this will extract the last one. Multiple values for utm_ keys
        should not appear in valid querystrings, so this may have an
        unpredictable outcome. Look after your querystrings.

        """
        return {str(k): str(v) for k, v in request.GET.items() if k.startswith("utm_")}


class LeadSourceMiddleware:
    """
    Store LeadSource and clear Session UTM values.

    If there are any utm_ params stored in the request session, this middleware
    will create a new LeadSource object from them, and clear out the values. This
    middleware should come after UtmSessionMiddleware.

    Only authenticated users have a LeadSource created - if the user is anonymous
    then the params are left in the request.session. If the session expires, or
    is deleted then this will be lost. If the user subsequently logs in, or is
    registered, then the session data will be retained, and on the first request
    with an authenticated user the data will be stored.

    """

    def __init__(self, get_response: Callable[[HttpRequest], HttpResponse]):
        self.get_response = get_response

    def has_utm(self, request: HttpRequest) -> bool:
        """Return True if the request has valid UTM params in session."""
        return all(
            [
                request.session.get("utm_medium", ""),
                request.session.get("utm_source", ""),
            ]
        )

    def capture_lead_source(self, request: HttpRequest) -> None:
        """
        Capture a new LeadSource from request UTM values.

        This method 'pop's the values from the session - so they will be deleted
        from hereon.

        """
        try:
            LeadSource.objects.create(
                user=request.user,
                medium=request.session.pop("utm_medium"),
                source=request.session.pop("utm_source"),
                campaign=request.session.pop("utm_campaign", ""),
                term=request.session.pop("utm_term", ""),
                content=request.session.pop("utm_content", ""),
            )
        except Exception as ex:  # pylint: disable=broad-except
            logger.warning("Error creating LeadSource: %s", ex)

    def __call__(self, request: HttpRequest) -> HttpResponse:
        if request.user.is_authenticated and self.has_utm(request):
            self.capture_lead_source(request)
        return self.get_response(request)



from django.conf import settings
from django.db import models
from django.utils import timezone


class LeadSource(models.Model):
    """
    Model used to track inbound leads.

    The model is separate from the ClientOnboarding and FreelancerOnboarding
    models so that we can track both in one place, and also so that we can
    backfill all historical data (before those models existed).

    NB The User field is *not* unique - it is possible for one User to have
    come through multiple routes (i.e. seen multiple ads); how we determine
    which ad is the one that made them register is a point of debate - so
    in this model we just record the data - and let the analysis happen later.

    The fields in this model are based on the industry standard UTM fields (
    see https://en.wikipedia.org/wiki/UTM_parameters for details).

    Most of the fields are optional, and included for completeness. Internal
    referrals do not have utm_* values associated with them, but the fields
    can be reused to identify the referrer.

    For example:

        medium: "referral"
        source: "internal"
        campaign: "profile" (e.g. book me)
        term: {{ referral token }}
        content: "book me"

    We are not using choices in this model as we can't predict what the params
    coming in from external sources may be, and we can't restrict the marketing
    team to specific values.

    """

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name="lead_sources",
    )
    medium = models.CharField(
        max_length=10,
        help_text=(
            "utm_medium: Identifies what type of link was used, "
            "such as cost per click or email."
        ),
    )
    source = models.CharField(
        max_length=30,
        help_text=(
            "utm_source: Identifies which site sent the traffic, "
            "and is a required parameter."
        ),
    )
    campaign = models.CharField(
        max_length=100,  # can be autogenerated by email campaigns
        help_text=(
            "utm_campaign: Identifies a specific product promotion "
            "or strategic campaign."
        ),
        blank=True,
    )
    term = models.CharField(
        max_length=50, help_text="utm_term: Identifies search terms.", blank=True
    )
    content = models.CharField(
        max_length=50,
        help_text=(
            "utm_content: Identifies what specifically was clicked to bring "
            "the user to the site, such as a banner ad or a text link."
        ),
        blank=True,
    )
    timestamp = models.DateTimeField(
        default=timezone.now, help_text="When the event occurred."
    )
    created_at = models.DateTimeField(
        default=timezone.now, help_text="When the event was recorded."
    )

    class Meta:
        get_latest_by = ("timestamp",)

    def __str__(self) -> str:
        return f"Lead source {self.id} for {self.user}: {self.medium}/{self.source}"

    def __repr__(self) -> str:
        return (
            f"<LeadSource id={self.id} user={self.user_id} "
            f"medium='{self.medium}' source='{self.source}'>"
        )



from django.contrib import admin

from .models import LeadSource


class LeadSourceAdmin(admin.ModelAdmin):
    raw_id_fields = ("user",)
    list_display = ("user", "medium", "source", "campaign", "timestamp")
    search_fields = ("user__first_name", "user__last_name")
    list_filter = ("medium", "source", "timestamp")
    readonly_fields = ("created_at", "timestamp")


admin.site.register(LeadSource, LeadSourceAdmin)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['utm_tracker', 'utm_tracker.migrations']

package_data = \
{'': ['*']}

install_requires = \
['django>=2.2,<4.0']

setup_kwargs = {
    'name': 'django-utm-tracker',
    'version': '0.1.1',
    'description': 'Django app for extracting and storing UTM tracking values.',
    'long_description': '# Django UTM Tracker\n\nDjango app for extracting and storing UTM tracking values.\n\n## Background\n\nThis app has been designed to integrate the standard `utm_*` querystring parameters\nthat are used by online advertisers with your Django project.\n\nIt does _not_ replace analytics (e.g. Google Analytics) and Adwords tracking, but does\nhave one crucial difference - it allows you to assign a specific user to a campaign\nadvert.\n\nThis may be useful if you are trying to assess the value of multiple channels /\ncampaigns.\n\n## How it works\n\nThe app works as a pair of middleware classes, that extract `utm_` values from any\nincoming request querystring, and then store those parameters against the request.user\n(if authenticated), or in the request.session (if not).\n\nThe following shows this workflow (pseudocode - see `test_utm_and_lead_source` for a\nreal example):\n\n```python\nclient = Client()\n# first request stashes values, but does not create a LeadSource as user is anonymous\nclient.get("/?utm_medium=medium&utm_source=source...")\nassert utm_values_in_session\nassert LeadSource.objects.count() == 0\n\n# subsequent request, with authenticated user, extracts values and stores LeadSource\nuser = User.objects.create(username="fred")\nclient.force_login(user, backend=settings.FORCED_AUTH_BACKEND)\nclient.get("/")\nassert not utm_values_in_session\nassert LeadSource.objects.count() == 1\n```\n\n### Why split the middleware in two?\n\nBy splitting the middleware into two classes, we enable the use case where we can\ntrack leads without `utm_` querystring parameters. For instance, if you have an\ninternal referral program, using a simple token, you can capture this as a\n`LeadSource` by adding sentinel values to the `request.session`:\n\n```python\ndef referral(request, token):\n    # do token handling\n    ...\n    # medium and source are mandatory for lead source capture\n    request.session["utm_medium"] = "referral"\n    request.session["utm_source"] = "internal"\n    # campaign, term and content are optional fields\n    request.session["utm_campaign"] = "july"\n    request.session["utm_term"] = token\n    request.session["utm_content"] = "buy-me"\n    return render(request, "landing_page.html")\n```\n\n## Configuration\n\nAdd the app to `INSTALLED_APPS`, and add both middleware classes to `MIDDLEWARE`.\n\nThe `UtmSession` middleware must come before `LeadSource` middleware.\n',
    'author': 'YunoJuno',
    'author_email': 'code@yunojuno.com',
    'maintainer': 'YunoJuno',
    'maintainer_email': 'code@yunojuno.com',
    'url': 'https://github.com/yunojuno/django-utm-tracker',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
